@@ -6,6 +6,7 @@ use Thermo_Routine
 use Math_Const
 use Phys_Const
 use Thermo_Advanced_Function
+use analy
 
 contains
 
@@ -28,13 +29,15 @@ subroutine Rich_horizon( za, pta, ptg, va, qva, qvs, Ri )
   nx=size(pta,1)
   ny=size(pta,2)
 
-!$omp parallel default(shared) do private(i,j)
+!$omp parallel default(shared)
+!$omp do private(i,j)
   do j=1,ny
      do i=1,nx
         Ri(i,j)=Rich( za, pta(i,j), ptg(i,j), va(i,j), qva(i,j), qvs(i,j) )
      end do
   end do
-!$omp end parallel do
+!$omp end do
+!$omp end parallel
 
 end subroutine
 
@@ -56,13 +59,15 @@ subroutine Louis_horizon( z, z0m, richard, Lo )
   nx=size(z0m,1)
   ny=size(z0m,2)
 
-!$omp parallel default(shared) do private(i,j)
+!$omp parallel default(shared)
+!$omp do private(i,j)
   do j=1,ny
      do i=1,nx
         Lo(i,j)=Louis( z, z0m(i,j), richard(i,j) )
      end do
   end do
-!$omp end parallel do
+!$omp end do
+!$omp end parallel
 
 end subroutine
 
@@ -83,21 +88,25 @@ subroutine cm_horizon( z, z0m, coem, richard )
   ny=size(z0m,2)
 
   if(present(richard))then
-!$omp parallel default(shared) do private(i,j)
+!$omp parallel default(shared)
+!$omp do private(i,j)
      do j=1,ny
         do i=1,nx
            coem(i,j)=cm( z, z0m(i,j), richard(i,j) )
         end do
      end do
-!$omp end parallel do
+!$omp end do
+!$omp end parallel
   else
-!$omp parallel default(shared) do private(i,j)
+!$omp parallel default(shared)
+!$omp do private(i,j)
      do j=1,ny
         do i=1,nx
            coem(i,j)=cm( z, z0m(i,j), richard(i,j) )
         end do
      end do
-!$omp end parallel do
+!$omp end do
+!$omp end parallel
   end if
 
 end subroutine
@@ -113,17 +122,67 @@ subroutine ust_horizon( cm, va, velst )
   real, intent(inout), dimension(size(cm,1),size(cm,2)) :: velst  ! ໤® [m/s]
   integer :: i, j, nx, ny
 
-!$omp parallel default(shared) do private(i,j)
+!$omp parallel default(shared)
+!$omp do private(i,j)
      do j=1,ny
         do i=1,nx
            velst(i,j)=ust( cm(i,j), va(i,j) )
         end do
      end do
-!$omp end parallel do
+!$omp end do
+!$omp end parallel
+
+end subroutine
+
+!--------------------------------------------------------
+
+subroutine Ertel_PV( nx, ny, nz, dx, dy, dz, u, v, w, rho, pt, pv )
+! 
+  implicit none
+  integer, intent(in) :: nx  !  1 ǿ
+  integer, intent(in) :: ny  !  2 ǿ
+  integer, intent(in) :: nz  !  3 ǿ
+  real, intent(in) :: dx  ! x γʻҴֳ [m]
+  real, intent(in) :: dy  ! y γʻҴֳ [m]
+  real, intent(in) :: dz  ! z γʻҴֳ [m]
+  real, intent(in) :: u(nx,ny,nz)  ! ®پ x ʬ [m/s]
+  real, intent(in) :: v(nx,ny,nz)  ! ®پ y ʬ [m/s]
+  real, intent(in) :: w(nx,ny,nz)  ! ®پ z ʬ [m/s]
+  real, intent(in) :: rho(nx,ny,nz)  ! ̩پ [kg/m^3]
+  real, intent(in) :: pt(nx,ny,nz)  ! ̾ [K]
+  real, intent(inout) :: pv(nx,ny,nz)  ! PV [Km^2/kgs]
+  real :: tmp1(nx,ny,nz)
+  real :: tmp2(nx,ny,nz)
+  real :: tmp3(nx,ny,nz)
+  real :: tmp4(nx,ny,nz)
+  real :: tmp5(nx,ny,nz)
+  real :: tmp6(nx,ny,nz)
+  real :: tmp7(nx,ny,nz)
+  integer :: i, j, k
+
+!  ̤ζָۤ׻.
+  call grads_vec_3d( nx, ny, nz, dx, dy, dz, pt, tmp1, tmp2, tmp3 )
+!  3  rotation ׻.
+  call rot( nx, ny, nz, dx, dy, dz, u, v, w, tmp4, tmp5, tmp6 )
+!  omega  grad pt Ѥ׻
+  call dot_prod( tmp4, tmp5, tmp6, tmp1, tmp2, tmp3, tmp7 )
+
+!$omp parallel default(shared)
+!$omp do private(i,j,k)
+
+  do k=1,nz
+     do j=1,ny
+        do i=1,nx
+           pv(i,j,k)=tmp7(i,j,k)/rho(i,j,k)
+        end do
+     end do
+  end do
+
+!$omp end do
+!$omp end parallel
 
 end subroutine
 
 
 
-
 end module
